'use strict';

var centroid = require('@turf/centroid').default;
var bbox = require('@turf/bbox').default;
var projection = require('@turf/projection');
var earcut = require('earcut');
var path = require('path');

module.exports = function (geojson, options) {
    var output = './output.stl';
    if (options && options.output != null) output = options.output;
    var extrude = 1;
    if (options && options.extrude != null) extrude = options.extrude;
    var size = 200;
    if (options && options.size != null) size = options.size;
    var projected = projection.toMercator(geojson);
    var scaled = scaleFeatures(projected, size);
    var facets = '';
    for (var i = 0; i < scaled.features.length; i++) {
        var feature = scaled.features[i].geometry.coordinates[0];
        facets += convertFeature(feature, extrude, size);
    }
    var filename = path.parse(output).base.split('.')[0];
    return combineStl(filename, facets);
};

var convertFeature = function convertFeature(feature, extrusion, size) {
    var stl = '';
    var bottom = [];
    var top = [];
    for (var i = 0; i < feature.length; i++) {
        bottom.push(feature[i].concat(0));
        top.push(feature[i].concat(extrusion));
    }
    var lines = [];
    for (var _i = 0; _i < feature.length; _i++) {
        var line = [feature[_i], feature[_i + 1]];
        if (_i + 1 != feature.length) {
            lines.push(line);
        }
    }
    for (var _i2 = 0; _i2 < lines.length; _i2++) {
        var facet = createFace(lines[_i2], extrusion);
        stl += facet;
    }
    stl += triangulate([top]);
    stl += triangulate([bottom]);
    return stl;
};

var triangulate = function triangulate(face) {
    var flat = earcut.flatten(face);
    var tris = earcut(flat.vertices, flat.holes, flat.dimensions);
    var arrays = [];
    var size = 3;
    while (tris.length > 0) {
        arrays.push(tris.splice(0, size));
    }
    var triangles = '';
    for (var i = 0; i < arrays.length; i++) {
        var facet = stlFacet(face[0][arrays[i][0]], face[0][arrays[i][1]], face[0][arrays[i][2]]);
        triangles += facet;
    }
    return triangles;
};

var scaleFeatures = function scaleFeatures(feature, size) {
    var featureCentroid = centroid(feature).geometry.coordinates;
    var dims = bbox(feature);
    var xDiff = dims[2] - dims[0];
    var yDiff = dims[3] - dims[1];
    var xDim = void 0,
        yDim = void 0;
    if (xDiff > yDiff) {
        xDim = size;
        yDim = size * yDiff / xDiff;
    } else if (yDiff > xDiff) {
        xDim = size * xDiff / yDiff;
        yDim = size;
    } else {
        xDim = size;
        yDim = size;
    }
    var xSize = xDim / xDiff;
    var ySize = yDim / yDiff;
    var offset = [-featureCentroid[0], -featureCentroid[1]];
    for (var i = 0; i < feature.features.length; i++) {
        var coordsOffset = [];
        for (var j = 0; j < feature.features[i].geometry.coordinates[0].length; j++) {
            var coords = feature.features[i].geometry.coordinates[0][j];
            var newCoords = [(coords[0] + offset[0]) * xSize, (coords[1] + offset[1]) * ySize];
            coordsOffset.push(newCoords);
        }
        feature.features[i].geometry.coordinates = [coordsOffset];
    }
    return feature;
};

var createFace = function createFace(line, extrusion) {
    var start = line[0];
    var end = line[1];
    var triangle1 = stlFacet(start.concat(0), start.concat(extrusion), end.concat(extrusion));
    var triangle2 = stlFacet(start.concat(0), end.concat(0), end.concat(extrusion));
    var face = '' + triangle1 + triangle2;
    return face;
};

var stlFacet = function stlFacet(point1, point2, point3) {
    var loop = 'facet normal 1.0 1.0 1.0 \n        outer loop\n            vertex ' + point1[0] + ' ' + point1[1] + ' ' + point1[2] + '\n            vertex ' + point2[0] + ' ' + point2[1] + ' ' + point2[2] + '\n            vertex ' + point3[0] + ' ' + point3[1] + ' ' + point3[2] + '\n        endloop\n    endfacet\n';
    return loop;
};

var combineStl = function combineStl(name, facets) {
    var template = 'solid ' + name + '\n' + facets;
    return template;
};